package org.kantega.jexmec.store.xml;

/*
 * Copyright 2009 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.kantega.jexmec.store.PluginStore;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

/**
 * Plugin store saving properties as child elements to an XML DOM element
 */
public abstract class XmlPluginStore implements PluginStore {
    private Element element;
    private static final String PROPERTY_ELEM = "property";
    private static final String NAME_ATTR = "name";

    public XmlPluginStore(Element element) {
        this.element = element;
    }

    public String getString(String key, String defaultValue) {
        synchronized (element.getOwnerDocument()) {
            Element propertyElement = getPropertyElement(key);

            return propertyElement != null ? getText(propertyElement) : defaultValue;

        }

    }

    private Element getPropertyElement(String key) {
        final NodeList props = element.getElementsByTagName(PROPERTY_ELEM);
        for(int i = 0; i < props.getLength(); i++) {
            Element prop = (Element) props.item(i);
            if(prop.getAttribute(NAME_ATTR) != null && prop.getAttribute(NAME_ATTR).equals(key)) {
                return prop;
            }

        }
        return null;
    }

    public void setString(String key, String value) {
        synchronized (element.getOwnerDocument()) {
            if(value != null) {
                Element propertyElement = getPropertyElement(key);
                if(propertyElement == null) {
                    propertyElement = element.getOwnerDocument().createElement(PROPERTY_ELEM);
                    propertyElement.setAttribute(NAME_ATTR, key);
                    element.appendChild(propertyElement);
                }
                while(propertyElement.getChildNodes().getLength() > 0) {
                    propertyElement.removeChild(propertyElement.getFirstChild());
                }
                propertyElement.appendChild(element.getOwnerDocument().createTextNode(value));
            } else {
                removeProperty(key);
            }
            propertySaved();
        }
    }

    private void removeProperty(String key) {
        final NodeList props = element.getElementsByTagName(PROPERTY_ELEM);
        for(int i = 0; i < props.getLength(); i++) {
            Element e = (Element) props.item(i);
            if(e.getAttribute(NAME_ATTR) != null && e.getAttribute(NAME_ATTR).equals(key) ) {
                e.getParentNode().removeChild(e);
                break;
            }
        }

    }

    private String getText(Element prop) {
        StringBuffer b = new StringBuffer();
        for(int i = 0; i < prop.getChildNodes().getLength(); i++) {
            final Node possibleTextNode = prop.getChildNodes().item(i);
            if(possibleTextNode instanceof Text) {
                b.append(((Text)possibleTextNode).getData());
            }
        }
        return b.toString();
    }

    abstract void propertySaved();
}
